@@ -32,7 +32,7 @@ struct fixed_rounding_class: function_class
   direction_type type;
   std::string ident;
   fixed_rounding_class(fixed_format const &f, direction_type t, std::string const &i)
-    : function_class(UOP_ID, TH_RND | TH_ABS | (t == ROUND_ZR || t == ROUND_AW ? TH_ABS_EXA_BND | TH_ABS_APX_BND : 0)),
+    : function_class(TH_RND | TH_ABS | (t == ROUND_ZR || t == ROUND_AW ? TH_ABS_EXA_BND | TH_ABS_APX_BND : 0)),
       format(f), type(t), ident(i) {}
   virtual interval round                         (interval const &, std::string &) const;
   virtual interval absolute_error                                  (std::string &) const;
@@ -151,8 +151,7 @@ proof_scheme *fixed_of_fix_scheme::factory(predicated_real const &real, ast_real
 {
   ast_real const *r = holders[1];
   real_op const *o = boost::get<real_op const>(holders[0]);
-  if (!o || !o->fun || o->fun->type != UOP_ID || r != o->ops[0])
-    return NULL;
+  if (!o || !o->fun || r != o->ops[0]) return NULL;
   fixed_rounding_class const *f = dynamic_cast<fixed_rounding_class const *>(o->fun);
   if (!f) return NULL;
   return new fixed_of_fix_scheme(real, predicated_real(r, PRED_FIX), f->format.min_exp);
