@@ -71,7 +71,7 @@ struct float_rounding_class: function_class
   std::string rname, ident;
   float_rounding_class(float_format const &f, direction_type t,
                        std::string const &r, std::string const &i)
-    : function_class(UOP_ID, TH_RND | TH_ENF |
+    : function_class(TH_RND | TH_ENF |
         (f.min_exp == INT_MIN ? TH_REL : (TH_REL_EXA_ABS | TH_REL_APX_ABS)) |
         (use_abs(t) ?  TH_ABS_EXA_ABS | TH_ABS_APX_ABS : TH_ABS_EXA_BND | TH_ABS_APX_BND)),
       format(f), type(t), rname(r), ident(i) {}
@@ -329,8 +329,7 @@ proof_scheme *float_of_fix_flt_scheme::factory(predicated_real const &real, ast_
 {
   ast_real const *r = holders[1];
   real_op const *o = boost::get<real_op const>(holders[0]);
-  if (!o || !o->fun || o->fun->type != UOP_ID || r != o->ops[0])
-    return NULL;
+  if (!o || !o->fun || r != o->ops[0]) return NULL;
   float_rounding_class const *f = dynamic_cast<float_rounding_class const *>(o->fun);
   if (!f || f->format.min_exp == INT_MIN) return NULL;
   preal_vect needed;
@@ -356,8 +355,7 @@ proof_scheme *floatx_of_flt_scheme::factory(predicated_real const &real, ast_rea
 {
   ast_real const *r = holders[1];
   real_op const *o = boost::get<real_op const>(holders[0]);
-  if (!o || !o->fun || o->fun->type != UOP_ID || r != o->ops[0])
-    return NULL;
+  if (!o || !o->fun || r != o->ops[0]) return NULL;
   float_rounding_class const *f = dynamic_cast<float_rounding_class const *>(o->fun);
   if (!f || f->format.min_exp != INT_MIN) return NULL;
   preal_vect needed;
