@@ -33,7 +33,7 @@ struct backend
   virtual std::string rewrite(ast_real const *, ast_real const *, pattern_cond_vect const &) = 0;
   virtual std::string theorem(node const *) = 0;
   virtual void finalize() = 0;
-  virtual ~backend() {}
+  virtual ~backend() = default;
   virtual bool is_known(std::string const &) { return true; }
   static backend *find(std::string const &);
 };
