@@ -129,12 +129,15 @@ static char const *theorem_defs[][2] = {
   { "neg_nzr", "$gpred_nzr.$t $1x $1p" },
   { "mul_nzr", "$gpred_nzr.$t $1x $2x $!" },
   { "div_nzr", "$gpred_nzr.$t $1x $2x $!" },
+  { "div_nzr_rev_l", "$gpred_nzr.$t $x _ $1p" },
+  { "div_nzr_rev_r", "$gpred_nzr.$t _ $x $1p" },
   { "nzr_of_abs", "$gpred_nzr.$t $x $1i $" },
   { "nzr_of_nzr_rel", "$gpred_nzr.$t $x $1x $2i $" },
   { "nzr_of_nzr_rel_rev", "$gpred_nzr.$t $1x $x $2i $!" },
 
   { "rel_refl", "$gpred_rel.$t $x $i $" },
   { "bnd_of_nzr_rel", "$gpred_rel.$t $2x $1x $i $!" },
+  { "bnd_of_rel", "$gpred_rel.$t $1x $1y $i $" },
   { "rel_of_nzr_bnd", "$gpred_rel.$t $x $1x $2i $!" },
   { "error_of_rel_pp", "$gpred_rel.$t $1x $2x $1i $2i $i $" },
   { "error_of_rel_po", "$gpred_rel.$t $1x $2x $1i $2i $i $" },
@@ -154,6 +157,7 @@ static char const *theorem_defs[][2] = {
 
   { "lin_of_bnd_div", "$gpred_lin.$t $x $y $i $2!" },
   { "bnd_div_of_lin", "$gpred_lin.$t $2x $2y $2i $!" },
+  { "bnd_of_lin", "$gpred_lin.$t $1x $1y $i $" },
   { "lin_of_rel", "$gpred_lin.$t $x $y $1i $i $" },
   { "add_ll", "$gpred_lin.$t $1x $2x $y $1i $2i $i $" },
   { "sub_ll", "$gpred_lin.$t $1x $2x $y $1i $2i $i $" },
@@ -311,11 +315,19 @@ static char const *theorem_defs[][2] = {
   { "div_firq", "$grewriting.$t $1x _ $1y $i $1p" },
 
   { "opp_fibm", "$grewriting.$t $1x $1y $i $1p" },
+  { "add_mibm", "$grewriting.$t _ _ _ _ $y $i $1p" },
+  { "add_film", "$grewriting.$t _ _ _ $y $i $1p" },
+  { "add_firm", "$grewriting.$t _ _ _ $y $i $1p" },
+  { "sub_mibm", "$grewriting.$t _ _ _ _ $y $i $1p" },
+  { "sub_film", "$grewriting.$t _ _ _ $y $i $1p" },
+  { "sub_firm", "$grewriting.$t _ _ _ $y $i $1p" },
+  { "sub_xalm", "$grewriting.$t _ _ _ $y $i $1p" },
   { "mul_film", "$grewriting.$t _ $1x $1y $i $1p" },
   { "mul_firm", "$grewriting.$t $1x _ $1y $i $1p" },
   { "mul_fism", "$grewriting.$t _ $1x $1y $i $1p" },
   { "mul_fitm", "$grewriting.$t $1x _ $1y $i $1p" },
   { "div_firm", "$grewriting.$t $1x _ $1y $i $1p" },
+  { "val_xabm", "$grewriting.$t _ $x $y $i $1p" },
 
   { "opp_fibe", "$grewriting.$t $1x $1y $1p" },
   { "add_file", "$grewriting.$t _ $1x $1y $1p" },
@@ -327,15 +339,16 @@ static char const *theorem_defs[][2] = {
   { "div_file", "$grewriting.$t _ $1x $1y $1p" },
   { "div_fire", "$grewriting.$t $1x _ $1y $1p" },
 
-  { NULL, NULL }
+  { "lin_of_rel2", "$grewriting.$t $1x $y $i $1p" },
 };
 
 theorem_map theorems;
 
 RUN_ONCE(fill_theorem_map)
 {
-  for (char const *(*p)[2] = theorem_defs; (*p)[0]; ++p) {
-    theorems.insert(theorem_map::value_type((*p)[0], (*p)[1]));
+  for (auto p: theorem_defs) {
+    auto v = theorems.insert(theorem_map::value_type(p[0], p[1]));
+    assert(v.second);
   }
 };
 
@@ -473,13 +486,12 @@ std::string display(ast_real const *r)
   else if (real_op const *o = boost::get< real_op const >(r))
   {
     static char const op[] = "X-XX+-*/XX";
-    if (o->type == ROP_FUN)
+    if (o->type == UOP_RND)
     {
       std::string description = o->fun->description();
-      plouf << convert_name(description) << ' ' << display(o->ops[0]);
-      for (ast_real_vect::const_iterator i = ++(o->ops.begin()),
-           end = o->ops.end(); i != end; ++i)
-        plouf << ' ' << display(*i);
+      plouf << convert_name(description);
+      for (ast_real const *i: o->ops)
+        plouf << ' ' << display(i);
     }
     else if (fqn)
     {
