@@ -130,7 +130,7 @@ static std::string display(ast_real const *r)
   else if (real_op const *o = boost::get< real_op const >(r))
   {
     static char const op[] = "X-XX+-*/XX";
-    if (o->type == ROP_FUN) {
+    if (o->type == UOP_RND) {
       plouf << convert_name(o->fun->description());
       for(ast_real_vect::const_iterator i = o->ops.begin(), end = o->ops.end(); i != end; ++i)
         plouf << " (" << display(*i) << ":real)";
