@@ -78,7 +78,7 @@ static std::string display(ast_real const *r, unsigned prio = 0)
   }
   if (real_op const *o = boost::get< real_op const >(r))
   {
-    if (o->type == ROP_FUN)
+    if (o->type == UOP_RND)
     {
       std::ostringstream s;
       s << "\\mathrm{" << convert_name(o->fun->pretty_name()) << "}(";
