@@ -54,7 +54,7 @@ class gs_rounding {
   void roundNA(mpfr_t &f) const { round(f, &gs_rounding::rndNA, &gs_rounding::rndNA); }
   void roundNU(mpfr_t &f) const { round(f, &gs_rounding::rndNZ, &gs_rounding::rndNA); }
   void roundND(mpfr_t &f) const { round(f, &gs_rounding::rndNA, &gs_rounding::rndNZ); }
-  virtual ~gs_rounding() {}
+  virtual ~gs_rounding() = default;
 };
 
 struct number;
