@@ -21,7 +21,6 @@
 #include "parser/ast.hpp"
 #include "proofs/schemes.hpp"
 
-extern bool parameter_rfma;
 link_map accurates, approximates;
 real_set user_reals;
 
@@ -130,18 +129,6 @@ void set_flags(ast_real *r, real_op const *p)
   r->has_placeholder = set_flags_placeholder(p);
 }
 
-ast_real const *unround(real_op_type type, ast_real_vect const &v) {
-  switch (type) {
-  case UOP_ID: return v[0];
-  case COP_FMA:
-    if (!parameter_rfma)
-      return normalize(ast_real(real_op(normalize(ast_real(real_op(v[0], BOP_MUL, v[1]))), BOP_ADD, v[2])));
-    else
-      return normalize(ast_real(real_op(v[2], BOP_ADD, normalize(ast_real(real_op(v[0], BOP_MUL, v[1]))))));
-  default: return normalize(ast_real(real_op(type, v)));
-  }
-}
-
 static static_ptr< cache<ast_ident> > ast_ident_cache;
 
 ast_ident *ast_ident::find(std::string const &s)
@@ -164,7 +151,7 @@ static void generate_approx_aux(
 {
   if (i == ops1.size())
   {
-    ast_real const *r2 = type == UOP_ID ? ops2[0] :
+    ast_real const *r2 = type == UOP_RND ? ops2[0] :
       ast_real_cache->test(ast_real(real_op(type, ops2)));
     if (!r2 || !r2->is_userdef) return;
     register_approx(r, r2);
@@ -189,8 +176,7 @@ static void generate_approx(ast_real const *r, real_set &already_done)
     generate_approx(j, already_done);
   }
   ast_real_vect ops = o->ops;
-  real_op_type t = o->type == ROP_FUN ? o->fun->type : o->type;
-  generate_approx_aux(r, t, 0, o->ops, ops);
+  generate_approx_aux(r, o->type, 0, o->ops, ops);
 }
 
 void generate_all_approx()
@@ -212,8 +198,7 @@ ast_real *normalize(ast_real const &v, bool user)
   }
   real_op *o = boost::get< real_op >(p);
   if (!o || !o->fun) return p;
-  ast_real const *a = unround(o->fun->type, o->ops);
-  register_approx(p, a);
+  register_approx(p, o->ops[0]);
   return p;
 }
 
@@ -237,7 +222,7 @@ std::string dump_real(ast_real const *r, unsigned prio)
   }
   if (real_op const *o = boost::get< real_op const >(r))
   {
-    if (o->type == ROP_FUN)
+    if (o->type == UOP_RND)
     {
       std::ostringstream s;
       s << o->fun->pretty_name() << '(' << dump_real(o->ops[0], 0);
