@@ -80,7 +80,7 @@ struct function_generator {
   function_generator(bool) {}
   function_generator(const char *);
   virtual function_class const *operator()(function_params const &) const = 0;
-  virtual ~function_generator() {}
+  virtual ~function_generator() = default;
 };
 
 struct default_function_generator: function_generator {
