@@ -34,18 +34,17 @@ extern ast_number const *token_zero, *token_one;
 
 enum condition_type { COND_LT, COND_LE, COND_GT, COND_GE, COND_NE, COND_NZ };
 
-enum real_op_type { UOP_ID, UOP_NEG, UOP_SQRT, UOP_ABS, BOP_ADD, BOP_SUB, BOP_MUL, BOP_DIV, COP_FMA, ROP_FUN };
+enum real_op_type { UOP_RND, UOP_NEG, UOP_SQRT, UOP_ABS, BOP_ADD, BOP_SUB, BOP_MUL, BOP_DIV };
 
 struct interval;
 
 struct function_class {
-  real_op_type type;
   int theorem_mask;
   static const int
     TH_RND = 1, TH_ENF = 2, TH_ABS = 4, TH_REL = 8,
     TH_ABS_EXA_BND = 16,  TH_ABS_EXA_ABS = 32,  TH_ABS_APX_BND = 64,   TH_ABS_APX_ABS = 128,
     TH_REL_EXA_BND = 256, TH_REL_EXA_ABS = 512, TH_REL_APX_BND = 1024, TH_REL_APX_ABS = 2048;
-  function_class(real_op_type t, int m): type(t), theorem_mask(m) {}
+  function_class(int m): theorem_mask(m) {}
   virtual interval round                         (interval const &, std::string &) const;
   virtual interval enforce                       (interval const &, std::string &) const;
   virtual interval absolute_error                                  (std::string &) const;
@@ -60,7 +59,7 @@ struct function_class {
   virtual interval relative_error_from_approx_abs(interval const &, std::string &) const;
   virtual std::string description() const = 0;
   virtual std::string pretty_name() const = 0;
-  virtual ~function_class() {}
+  virtual ~function_class() = default;
 };
 
 struct ast_real;
@@ -74,8 +73,7 @@ struct real_op {
   real_op_type type;
   real_op(real_op_type t, ast_real_vect const &o): ops(o), fun(NULL), type(t) {}
   real_op(real_op_type t, function_class const *f, ast_real_vect const &o): ops(o), fun(f), type(t) {}
-  real_op(function_class const *f, ast_real_vect const &o): ops(o), fun(f), type(ROP_FUN) {}
-  real_op(function_class const *f, ast_real const *o): ops(1, o), fun(f), type(ROP_FUN) {}
+  real_op(function_class const *f, ast_real const *o): ops(1, o), fun(f), type(UOP_RND) {}
   real_op(real_op_type t, ast_real const *o): ops(1, o), fun(NULL), type(t) {}
   real_op(ast_real const *l, real_op_type t, ast_real const *r): fun(NULL), type(t) { ops.push_back(l); ops.push_back(r); }
   bool operator==(real_op const &v) const;
@@ -136,7 +134,6 @@ struct ast_real: ast_real_aux
 
 inline bool is_constant(ast_real const *r) { return r->is_constant; }
 ast_real *normalize(ast_real const &, bool user = false);
-ast_real const *unround(real_op_type, ast_real_vect const &);
 
 typedef std::map<ast_real const *, real_set> link_map;
 extern link_map accurates, approximates;
