@@ -288,13 +288,13 @@ function_class const *absolute_rounding_error(ast_real const *src, ast_real cons
   dst[0] = p->ops[1];
   dst[1] = p->ops[0];
   real_op const *o = boost::get< real_op const >(dst[1]);
-  if (!o || !o->fun) return NULL;
-  return (dst[0] != unround(o->fun->type, o->ops)) ? NULL : o->fun;
+  if (!o || !o->fun || dst[0] != o->ops[0]) return NULL;
+  return o->fun;
 }
 
 function_class const *relative_rounding_error(predicated_real const &src) {
   if (src.pred() != PRED_REL) return NULL;
   real_op const *o = boost::get< real_op const >(src.real());
-  if (!o || !o->fun) return NULL;
-  return (src.real2() != unround(o->fun->type, o->ops)) ? NULL : o->fun;
+  if (!o || !o->fun || src.real2() != o->ops[0]) return NULL;
+  return o->fun;
 }
