@@ -247,7 +247,7 @@ proof_scheme *rounding_bound_scheme::factory(ast_real const *real)
 {
   real_op const *p = boost::get < real_op const >(real);
   if (!p || !p->fun || !(p->fun->theorem_mask & function_class::TH_RND)) return NULL;
-  return new rounding_bound_scheme(real, unround(p->fun->type, p->ops), p->fun);
+  return new rounding_bound_scheme(real, p->ops[0], p->fun);
 }
 
 // BND(rnd(a)) <- BND(rnd(a))
@@ -288,6 +288,26 @@ proof_scheme *lin_of_rel_scheme::factory(predicated_real const &real, ast_real_v
   return new lin_of_rel_scheme(real, needed);
 }
 
+// LIN(a - b, b) <- REL(a, b)
+REGISTER_SCHEME_BEGIN(lin_of_rel2);
+lin_of_rel2_scheme(predicated_real const &r, preal_vect const &v) :
+  proof_scheme(r, v, "lin_of_rel2", UPD_COPY) {}
+REGISTER_SCHEME_END_PATTERN(lin_of_rel2, predicated_real(pattern(0) - pattern(1), pattern(1), PRED_LIN));
+
+void lin_of_rel2_scheme::compute(property const hyps[], property &res, std::string &) const {
+  res.bnd() = hyps[0].bnd();
+}
+
+proof_scheme *lin_of_rel2_scheme::factory(predicated_real const &real, ast_real_vect const &holders) {
+  ast_real const *a = holders[0];
+  ast_real const *b = holders[1];
+  if (!is_approx(a, b)) return NULL;
+
+  preal_vect needed;
+  needed.push_back(predicated_real(a, b, PRED_REL));
+  return new lin_of_rel2_scheme(real, needed);
+}
+
 // FLT(a - b) <- FLT(a), FLT(b), LIN(a, b)
 REGISTER_SCHEME_BEGIN(sub_flt_lin);
 sub_flt_lin_scheme(predicated_real const &r, preal_vect const &v) : proof_scheme(r, v, "sub_flt_lin") {}
@@ -337,7 +357,7 @@ proof_scheme *sub_flt_lin_rev_scheme::factory(predicated_real const &real, ast_r
 // FLT(a + b) <- FLT(a), FLT(b), LIN(a, b)
 REGISTER_SCHEME_BEGIN(add_flt_lin);
 add_flt_lin_scheme(predicated_real const &r, preal_vect const &v) : proof_scheme(r, v, "add_flt_lin") {}
-REGISTER_SCHEME_END_PATTERN(add_flt_lin, predicated_real(pattern(0) - pattern(1), PRED_FLT));
+REGISTER_SCHEME_END_PATTERN(add_flt_lin, predicated_real(pattern(0) + pattern(1), PRED_FLT));
 
 void add_flt_lin_scheme::compute(property const hyps[], property &res, std::string &) const {
   static interval r(-2, -0.5);
@@ -360,7 +380,7 @@ proof_scheme *add_flt_lin_scheme::factory(predicated_real const &real, ast_real_
 // FLT(a + b) <- FLT(a), FLT(b), LIN(b, a)
 REGISTER_SCHEME_BEGIN(add_flt_lin_rev);
 add_flt_lin_rev_scheme(predicated_real const &r, preal_vect const &v) : proof_scheme(r, v, "add_flt_lin_rev") {}
-REGISTER_SCHEME_END_PATTERN(add_flt_lin_rev, predicated_real(pattern(0) - pattern(1), PRED_FLT));
+REGISTER_SCHEME_END_PATTERN(add_flt_lin_rev, predicated_real(pattern(0) + pattern(1), PRED_FLT));
 
 void add_flt_lin_rev_scheme::compute(property const hyps[], property &res, std::string &) const {
   static interval r(-2, -0.5);
@@ -393,6 +413,10 @@ void lin_of_bnd_div_scheme::compute(property const hyps[], property &res, std::s
 proof_scheme *lin_of_bnd_div_scheme::factory(predicated_real const &real, ast_real_vect const &holders) {
   ast_real const *a = holders[0];
   ast_real const *b = holders[1];
+  if (real_op const *p = boost::get<real_op const>(a)) {
+    if (p->type == BOP_SUB && p->ops[1] == b && is_approx(p->ops[0], b))
+      return NULL;
+  }
 
   preal_vect needed;
   needed.push_back(predicated_real(b, PRED_NZR));
@@ -423,6 +447,28 @@ proof_scheme *bnd_div_of_lin_scheme::factory(predicated_real const &real, ast_re
   return new bnd_div_of_lin_scheme(real, needed);
 }
 
+// BND(a, b) <- LIN(a, b)
+REGISTER_SCHEME_BEGIN(bnd_of_lin);
+  bnd_of_lin_scheme(predicated_real const &r, preal_vect const &v)
+    : proof_scheme(r, v, "bnd_of_lin", UPD_COPY) {}
+REGISTER_SCHEME_END_PATTERN(bnd_of_lin, predicated_real(pattern(0) / pattern(1), PRED_BND));
+
+void bnd_of_lin_scheme::compute(property const hyps[], property &res, std::string &) const
+{
+  interval const &i = hyps[0].bnd();
+  if (!contains_zero(i)) return;
+  res.bnd() = i;
+}
+
+proof_scheme *bnd_of_lin_scheme::factory(predicated_real const &real, ast_real_vect const &holders)
+{
+  if (holders[0] == holders[1]) return NULL;
+  if (holders[0]->is_constant && holders[1]->is_constant) return NULL;
+  preal_vect hyps;
+  hyps.push_back(predicated_real(holders[0], holders[1], PRED_LIN));
+  return new bnd_of_lin_scheme(real, hyps);
+}
+
 #if 0
 // LIN(a, c) <- LIN(a, b), LIN(b, c)
 REGISTER_SCHEME_BEGIN(compose_lin);
@@ -448,66 +494,6 @@ proof_scheme *compose_lin_scheme::factory(predicated_real const &real, ast_real_
 }
 #endif
 
-// LIN(a + b, c) <- LIN(a, c), LIN(b, c)
-REGISTER_SCHEME_BEGIN(add_lin);
-add_lin_scheme(predicated_real const &r, preal_vect const &v)
-  : proof_scheme(r, v, "add_ll") {}
-REGISTER_SCHEME_END_PATTERN(add_lin, predicated_real(pattern(0) + pattern(1), pattern(2), PRED_LIN));
-
-void add_lin_scheme::compute(property const hyps[], property &res, std::string &) const {
-  res.bnd() = hyps[0].bnd() + hyps[1].bnd();
-}
-
-proof_scheme *add_lin_scheme::factory(predicated_real const &real, ast_real_vect const &holders) {
-  ast_real const *a = holders[0];
-  ast_real const *b = holders[1];
-  ast_real const *c = holders[2];
-
-  preal_vect needed;
-  needed.push_back(predicated_real(a, c, PRED_LIN));
-  needed.push_back(predicated_real(b, c, PRED_LIN));
-  return new add_lin_scheme(real, needed);
-}
-
-// LIN(a - b, c) <- LIN(a, c), LIN(b, c)
-REGISTER_SCHEME_BEGIN(sub_lin);
-sub_lin_scheme(predicated_real const &r, preal_vect const &v)
-  : proof_scheme(r, v, "sub_ll") {}
-REGISTER_SCHEME_END_PATTERN(sub_lin, predicated_real(pattern(0) - pattern(1), pattern(2), PRED_LIN));
-
-void sub_lin_scheme::compute(property const hyps[], property &res, std::string &) const {
-  res.bnd() = hyps[0].bnd() - hyps[1].bnd();
-}
-
-proof_scheme *sub_lin_scheme::factory(predicated_real const &real, ast_real_vect const &holders) {
-  ast_real const *a = holders[0];
-  ast_real const *b = holders[1];
-  ast_real const *c = holders[2];
-  if (b == c) return NULL;
-
-  preal_vect needed;
-  needed.push_back(predicated_real(a, c, PRED_LIN));
-  needed.push_back(predicated_real(b, c, PRED_LIN));
-  return new sub_lin_scheme(real, needed);
-}
-
-// LIN(-a, b) <- LIN(a, b)
-REGISTER_SCHEME_BEGIN(minus_lin);
-minus_lin_scheme(predicated_real const &r, preal_vect const &v) : proof_scheme(r, v, "minus_lin") {}
-REGISTER_SCHEME_END_PATTERN(minus_lin, predicated_real(-pattern(0), pattern(1), PRED_LIN));
-
-void minus_lin_scheme::compute(property const hyps[], property &res, std::string &) const
-{
-  res.bnd() = -hyps[0].bnd();
-}
-
-proof_scheme *minus_lin_scheme::factory(predicated_real const &real, ast_real_vect const &holders)
-{
-  preal_vect hyps;
-  hyps.push_back(predicated_real(holders[0], holders[1], PRED_LIN));
-  return new minus_lin_scheme(real, hyps);
-}
-
 #if 0
 // LIN(b - a) <- LIN(a - b)
 REGISTER_SCHEME_BEGIN(lin_of_opp_lin);
@@ -771,9 +757,8 @@ proof_scheme *computation_scheme::factory(ast_real const *real)
   if (ops.size() != 2 || ops[0] != ops[1]) {
     generic:
     needed.reserve(ops.size());
-    for (ast_real_vect::const_iterator i = ops.begin(), i_end = ops.end();
-         i != i_end; ++i)
-      needed.push_back(predicated_real(*i, PRED_BND));
+    for (ast_real const *i: p->ops)
+      needed.push_back(predicated_real(i, PRED_BND));
     switch (p->type) {
     case UOP_NEG: name = "neg"; break;
     case UOP_ABS: name = "abs"; break;
@@ -841,9 +826,8 @@ proof_scheme *computation_abs_scheme::factory(predicated_real const &real)
   if (is_constant(real.real())) return NULL;
   preal_vect needed;
   needed.reserve(p->ops.size());
-  for (ast_real_vect::const_iterator i = p->ops.begin(), i_end = p->ops.end();
-       i != i_end; ++i)
-    needed.push_back(predicated_real(*i, PRED_ABS));
+  for (ast_real const *i: p->ops)
+    needed.push_back(predicated_real(i, PRED_ABS));
   char const *name = "";
   update upd = UPD_SEEK;
   switch (p->type) {
@@ -856,6 +840,67 @@ proof_scheme *computation_abs_scheme::factory(predicated_real const &real)
   return new computation_abs_scheme(real, needed, name, upd);
 }
 
+// COMPUTATION_LIN
+REGISTER_SCHEME_BEGIN(computation_lin);
+  real_op const *naked_real;
+  computation_lin_scheme(predicated_real const &r, real_op const *nr, preal_vect const &v, char const *n)
+    : proof_scheme(r, v, n), naked_real(nr) {}
+REGISTER_SCHEME_END_PREDICATE(computation_lin);
+
+void computation_lin_scheme::compute(property const hyps[], property &res, std::string &) const
+{
+  real_op const *r = naked_real;
+  switch (r->ops.size()) {
+  case 1: {
+    interval const &i = hyps[0].bnd();
+    switch (r->type) {
+    case UOP_NEG:
+      res.bnd() = -i;
+      return;
+    default:
+      assert(false);
+    }
+    break; }
+  case 2: {
+    interval const &i1 = hyps[0].bnd();
+    interval const &i2 = hyps[1].bnd();
+    switch (r->type) {
+    case BOP_ADD: res.bnd() = i1 + i2; return;
+    case BOP_SUB: res.bnd() = i1 - i2; return;
+    default:
+      assert(false);
+    } }
+  default:
+    assert(false);
+  }
+}
+
+proof_scheme *computation_lin_scheme::factory(predicated_real const &real)
+{
+  if (real.pred() != PRED_LIN) return NULL;
+  ast_real const *r = real.real();
+  if (hidden_real const *h = boost::get<hidden_real const>(r))
+    r = h->real;
+  real_op const *p = boost::get<real_op const>(r);
+  if (!p || p->fun) return NULL;
+  ast_real_vect const &ops = p->ops;
+  preal_vect needed;
+  char const *name = "";
+  needed.reserve(ops.size());
+  for (ast_real const *i: p->ops)
+    needed.push_back(predicated_real(i, real.real2(), PRED_LIN));
+  switch (p->type) {
+  case UOP_NEG: name = "minus_lin"; break;
+  case BOP_ADD: name = "add_ll"; break;
+  case BOP_SUB:
+    if (p->ops[1] == real.real2()) return NULL;
+    name = "sub_ll";
+    break;
+  default: return NULL;
+  }
+  return new computation_lin_scheme(real, p, needed, name);
+}
+
 // ABS(a) <- BND(a * a)
 REGISTER_SCHEME_BEGIN(square_rev);
   square_rev_scheme(predicated_real const &r, ast_real const *u)
@@ -1217,7 +1262,30 @@ proof_scheme *bnd_of_nzr_rel_scheme::factory(predicated_real const &real, ast_re
   return new bnd_of_nzr_rel_scheme(real, hyps);
 }
 
-// REL(a, b) <- NZR(a), BND((a - b) / b)
+// BND((b - a) / a) <- REL(b, a)
+REGISTER_SCHEME_BEGIN(bnd_of_rel);
+  bnd_of_rel_scheme(predicated_real const &r, preal_vect const &v)
+    : proof_scheme(r, v, "bnd_of_rel", UPD_COPY) {}
+REGISTER_SCHEME_END_PATTERN(bnd_of_rel,
+  predicated_real((pattern(1) - pattern(0)) / pattern(0), PRED_BND));
+
+void bnd_of_rel_scheme::compute(property const hyps[], property &res, std::string &) const
+{
+  interval const &i = hyps[0].bnd();
+  if (!contains_zero(i)) return;
+  res.bnd() = i;
+}
+
+proof_scheme *bnd_of_rel_scheme::factory(predicated_real const &real, ast_real_vect const &holders)
+{
+  if (holders[0] == holders[1]) return NULL;
+  if (holders[0]->is_constant && holders[1]->is_constant) return NULL;
+  preal_vect hyps;
+  hyps.push_back(predicated_real(holders[1], holders[0], PRED_REL));
+  return new bnd_of_rel_scheme(real, hyps);
+}
+
+// REL(a, b) <- NZR(b), BND((a - b) / b)
 REGISTER_SCHEME_BEGIN(rel_of_nzr_bnd);
   rel_of_nzr_bnd_scheme(predicated_real const &r, preal_vect const &v)
     : proof_scheme(r, v, "rel_of_nzr_bnd", UPD_COPY) {}
@@ -1590,6 +1658,40 @@ proof_scheme *div_nzr_scheme::factory(predicated_real const &real, ast_real_vect
   return new div_nzr_scheme(real, hyps);
 }
 
+// NZR(a) <- NZR(a / b)
+REGISTER_SCHEME_BEGIN(div_nzr_rev_l);
+  div_nzr_rev_l_scheme(predicated_real const &r, preal_vect const &v)
+    : proof_scheme(r, v, "div_nzr_rev_l") {}
+REGISTER_SCHEME_END_PATTERN_USER(div_nzr_rev_l, predicated_real(pattern(0), PRED_NZR), pattern(0) / pattern(1));
+
+void div_nzr_rev_l_scheme::compute(property const[], property &, std::string &) const
+{
+}
+
+proof_scheme *div_nzr_rev_l_scheme::factory(predicated_real const &real, ast_real_vect const &holders)
+{
+  preal_vect hyps;
+  hyps.push_back(predicated_real(normalize(real_op(holders[0], BOP_DIV, holders[1])), PRED_NZR));
+  return new div_nzr_rev_l_scheme(real, hyps);
+}
+
+// NZR(b) <- NZR(a / b)
+REGISTER_SCHEME_BEGIN(div_nzr_rev_r);
+  div_nzr_rev_r_scheme(predicated_real const &r, preal_vect const &v)
+    : proof_scheme(r, v, "div_nzr_rev_r") {}
+REGISTER_SCHEME_END_PATTERN_USER(div_nzr_rev_r, predicated_real(pattern(1), PRED_NZR), pattern(0) / pattern(1));
+
+void div_nzr_rev_r_scheme::compute(property const[], property &, std::string &) const
+{
+}
+
+proof_scheme *div_nzr_rev_r_scheme::factory(predicated_real const &real, ast_real_vect const &holders)
+{
+  preal_vect hyps;
+  hyps.push_back(predicated_real(normalize(real_op(holders[0], BOP_DIV, holders[1])), PRED_NZR));
+  return new div_nzr_rev_r_scheme(real, hyps);
+}
+
 // NZR(a) <- ABS(a)
 REGISTER_SCHEME_BEGIN(nzr_of_abs);
   nzr_of_abs_scheme(predicated_real const &r, predicated_real const &n)
