@@ -46,7 +46,7 @@ struct splitter
    * @return false if all the intervals have been handled.
    */
   virtual number next(int &iter_max) = 0;
-  virtual ~splitter() {}
+  virtual ~splitter() = default;
   /**
    * Indicates whether intervals should be merged afterwards.
    * @return false by default.
