@@ -41,14 +41,13 @@ struct rewriting_scheme: proof_scheme
 {
   predicated_real rewritten;
   pattern_cond_vect conditions;
-  preal_vect needed_reals(predicated_real const &, pattern_cond_vect const &) const;
+  static preal_vect needed_reals(predicated_real const &, pattern_cond_vect const &);
   rewriting_scheme(predicated_real const &src, predicated_real const &dst, char const *n, pattern_cond_vect const &c)
     : proof_scheme(src, needed_reals(dst, c), n, UPD_COPY), rewritten(dst), conditions(c) {}
   virtual void compute(property const[], property &, std::string &) const;
 };
 
-preal_vect rewriting_scheme::needed_reals(predicated_real const &d,
-  pattern_cond_vect const &c) const
+preal_vect rewriting_scheme::needed_reals(predicated_real const &d, pattern_cond_vect const &c)
 {
   preal_vect res;
   for (pattern_cond const &i: c) {
@@ -274,7 +273,7 @@ pattern_excl_vect operator&&(pattern_excl_vect const &v, pattern_excl const &c)
   return res;
 }
 
-static pattern a(0), b(1), c(2), d(3), a_b(-1), one(token_one);
+static pattern a(0), b(1), c(2), d(3), e(4), a_b(-1), one(token_one);
 
 #define abs pattern::abs
 #define sqrt pattern::sqrt
@@ -353,16 +352,6 @@ REWRITe(add_firs,
 	a - c,
 	a ^ c);
 
-REWRITe(add_filq,
-	((a + b) - (a + c)) / (a + c),
-	(b - c) / (a + c),
-	b ^ c);
-
-REWRITe(add_firq,
-	((a + b) - (c + b)) / (c + b),
-	(a - c) / (c + b),
-	a ^ c);
-
 R3WRITE(add_xilu,
 	a,
 	hide((a + b) - b),
@@ -400,16 +389,6 @@ REWRITe(sub_firs,
 	a - c,
 	a ^ c);
 
-REWRITe(sub_filq,
-	((a - b) - (a - c)) / (a - c),
-	hide(-((b - c) / (a - c))),
-	b ^ c);
-
-REWRITe(sub_firq,
-	((a - b) - (c - b)) / (c - b),
-	(a - c) / (c - b),
-	a ^ c);
-
 R3WRITE(val_xabs, // actually sub_xilu
 	b,
 	hide(a + (b - a)),
@@ -474,11 +453,6 @@ R3WRIT3(mul_xiru,
 
 // DIV
 
-REWRITe(div_xals,
-	b / c,
-	hide((b - a) / c + a / c),
-	a ^ c && b ^ c);
-
 REWRIT3(div_fir,
 	(a * b) / b,
 	a,
@@ -489,13 +463,6 @@ REWRIT3(div_fil,
 	b,
 	~a);
 
-/*
-REWRITe(div_firs,
-	a / b - c / b,
-	hide((a - c) / b),
-	a ^ c);
-*/
-
 R3WRIT3(div_xilu,
 	a,
 	hide((a / b) * b),
@@ -657,6 +624,46 @@ REWRITe(div_firm,
 	a, c,
 	a ^ c);
 
+REWRITe(val_xabm,
+	b, c,
+	hide(a + (b - a)), c,
+	a ^ c && b ^ c);
+
+REWRITe(add_mibm,
+	(a + b) - (c + d), e,
+	hide((a - c) + (b - d)), e,
+	a ^ c && b ^ d);
+
+REWRITe(add_film,
+	(a + b) - (a + c), d,
+	b - c, d,
+	b ^ c);
+
+REWRITe(add_firm,
+	(a + b) - (c + b), d,
+	a - c, d,
+	a ^ c);
+
+REWRITe(sub_mibm,
+	(a - b) - (c - d), e,
+	hide((a - c) - (b - d)), e,
+	a ^ c && b ^ d);
+
+REWRITe(sub_film,
+	(a - b) - (a - c), e,
+	hide(-(b - c)), e,
+	b ^ c);
+
+REWRITe(sub_firm,
+	(a - b) - (c - b), e,
+	a - c, e,
+	a ^ c);
+
+REWRITe(sub_xalm,
+	b - c, d,
+	hide((b - a) + (a - c)), d,
+	a ^ c && b ^ c);
+
 #undef REWRITE
 #undef REWRITe
 
