@@ -1,6 +1,6 @@
 /*
    Copyright (C) 2004 - 2015 by Guillaume Melquiond <guillaume.melquiond@inria.fr>
-   Part of the Gappa tool http://gappa.gforge.inria.fr/
+   Part of the Gappa tool https://gappa.gitlabpages.inria.fr/
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the CeCILL Free Software License Agreement or
@@ -177,10 +177,7 @@ int stat_tested_real = 0, stat_discarded_real = 0,
 static void initialize_scheme_list(scheme_queue &v)
 {
   ++visit_counter;
-  for (scheme_vect::const_iterator i = source_schemes.begin(),
-       i_end = source_schemes.end(); i != i_end; ++i)
-  {
-    proof_scheme const *s = *i;
+  for (proof_scheme const *s: source_schemes) {
     s->visited = visit_counter;
     v.push(s);
   }
@@ -191,11 +188,8 @@ static void initialize_scheme_list(scheme_queue &v)
  */
 static void insert_dependent(scheme_queue &v, predicated_real const &real, proof_scheme const *ss = NULL)
 {
-  scheme_set const &dep = reals[real].dependent;
-  for (scheme_set::const_iterator i = dep.begin(),
-       i_end = dep.end(); i != i_end; ++i)
+  for (proof_scheme const *s: reals[real].dependent)
   {
-    proof_scheme const *s = *i;
     if (s == ss || !s->can_visit()) continue;
     if (s->hyp_cache) {
       unsigned l = s->needed_reals.size();
@@ -214,9 +208,7 @@ static void delete_scheme(proof_scheme const *s, predicated_real const *restrict
   // if there is a restriction, we are removing a scheme that depends on a real,
   // otherwise we are removing a scheme from a real; in both cases, we should
   // not modify the dependencies of this real
-  preal_vect v = s->needed_reals;
-  for(preal_vect::const_iterator i = v.begin(), end = v.end(); i != end; ++i) {
-    predicated_real const &real = *i;
+  for (predicated_real const &real: s->needed_reals) {
     if (restricted_real && real == *restricted_real) continue;
     reals[real].dependent.erase(s);
   }
@@ -256,11 +248,9 @@ static real_dependency &initialize_dependencies(predicated_real const &real)
     if (f.user_source)
     {
       // look for user reals to fill the pattern holes
-      for (real_set::const_iterator j = user_reals.begin(),
-           j_end = user_reals.end(); j != j_end; ++j)
-      {
+      for (ast_real const *j: user_reals) {
         ast_real_vect h = holders;
-        if (!match(*j, f.user_source, h)) continue;
+        if (!match(j, f.user_source, h)) continue;
         if (proof_scheme *s = f(real, h)) l.insert(s);
       }
       continue;
@@ -273,20 +263,16 @@ static real_dependency &initialize_dependencies(predicated_real const &real)
     link_map const *lm = p ? &approximates : &accurates;
     link_map::const_iterator k = lm->find(holders[1 - p]);
     if (k == lm->end()) continue;
-    real_set const &s = k->second;
-    for (real_set::const_iterator j = s.begin(), j_end = s.end(); j != j_end; ++j) {
-      holders[p] = *j;
+    for (ast_real const *j: k->second) {
+      holders[p] = j;
       if (proof_scheme *s = f(real, holders)) l.insert(s);
     }
   }
   // create the dependencies
-  for (scheme_set::const_iterator i = l.begin(), i_end = l.end(); i != i_end; ++i)
-  {
+  for (proof_scheme const *s: l) {
     ++stat_tested_theo;
-    proof_scheme const *s = *i;
-    preal_vect v = s->needed_reals;
-    for(preal_vect::const_iterator j = v.begin(), j_end = v.end(); j != j_end; ++j)
-      initialize_dependencies(*j).dependent.insert(s);
+    for (predicated_real const &j: s->needed_reals)
+      initialize_dependencies(j).dependent.insert(s);
   }
   return it->second;
 }
@@ -305,10 +291,7 @@ static void mark_dependent_schemes(predicated_real const &real)
   {
     predicated_real r = pending_reals.back();
     pending_reals.pop_back();
-    scheme_set const &v = reals[r].dependent;
-    for (scheme_set::const_iterator i = v.begin(), i_end = v.end(); i != i_end; ++i)
-    {
-      proof_scheme const *s = *i;
+    for (proof_scheme const *s: reals[r].dependent) {
       if (!s || !s->can_visit()) continue;
       if (reals[s->real].can_visit()) pending_reals.push_back(s->real);
     }
@@ -327,10 +310,8 @@ static void mark_useful_reals(predicated_real const &real)
   {
     predicated_real r = pending_reals.back();
     pending_reals.pop_back();
-    scheme_set const &v = reals[r].schemes;
-    for (scheme_set::const_iterator i = v.begin(), i_end = v.end(); i != i_end; ++i)
+    for (proof_scheme const *s: reals[r].schemes)
     {
-      proof_scheme const *s = *i;
       if (!s) continue;
       preal_vect w = s->needed_reals;
       for (preal_vect::const_iterator j = w.begin(), j_end = w.end(); j != j_end; ++j)
@@ -345,17 +326,13 @@ static void mark_useful_reals(predicated_real const &real)
 preal_vect generate_proof_paths()
 {
   // recursively search schemes needed for output reals
-  for (preal_set::const_iterator i = output_reals.begin(),
-       i_end = output_reals.end(); i != i_end; ++i)
-  {
-    real_dependency &r = initialize_dependencies(*i);
+  for (predicated_real const &i: output_reals) {
+    real_dependency &r = initialize_dependencies(i);
     r.dependent.insert(NULL);
   }
   // initialize hypothesis reals to handle contradictions
-  for (preal_set::const_iterator i = input_reals.begin(),
-       i_end = input_reals.end(); i != i_end; ++i)
-  {
-    real_dependency &r = initialize_dependencies(*i);
+  for (predicated_real const &i: input_reals) {
+    real_dependency &r = initialize_dependencies(i);
     r.schemes.insert(NULL);
     r.dependent.insert(NULL);
   }
@@ -363,50 +340,37 @@ preal_vect generate_proof_paths()
   ++visit_counter;
   for (real_dependencies::iterator i = reals.begin(), i_end = reals.end(); i != i_end; ++i)
   {
-    real_dependency &r = i->second;
-    for (scheme_set::iterator j = r.schemes.begin(), j_end = r.schemes.end(); j != j_end; ++j)
-    {
-      proof_scheme const *s = *j;
+    for (proof_scheme const *s: i->second.schemes) {
       if (!s || !s->needed_reals.empty()) continue;
       s->visited = visit_counter;
       mark_dependent_schemes(s->real);
     }
   }
-  for (preal_set::const_iterator i = input_reals.begin(),
-       i_end = input_reals.end(); i != i_end; ++i)
-  {
-    mark_dependent_schemes(*i);
-  }
+  for (predicated_real const &i: input_reals)
+    mark_dependent_schemes(i);
   for (real_dependencies::iterator i = reals.begin(), i_end = reals.end(); i != i_end; ++i)
   {
     real_dependency &r = i->second;
     scheme_set v;
     v.swap(r.schemes);
-    for (scheme_set::iterator j = v.begin(), j_end = v.end(); j != j_end; ++j)
-    {
-      proof_scheme const *s = *j;
+    for (proof_scheme const *s: v) {
       if (!s || s->visited == visit_counter) r.schemes.insert(s);
       else delete_scheme(s, NULL);
     }
   }
   // find reals reaching to outputs, then remove unneeded reals
   ++visit_counter;
-  for (preal_set::const_iterator i = output_reals.begin(),
-       i_end = output_reals.end(); i != i_end; ++i)
-  {
-    mark_useful_reals(*i);
-  }
-  for (preal_set::const_iterator i = input_reals.begin(),
-       i_end = input_reals.end(); i != i_end; ++i)
-  {
-    mark_useful_reals(*i);
-  }
+  for (predicated_real const &i: output_reals)
+    mark_useful_reals(i);
+  for (predicated_real const &i: input_reals)
+    mark_useful_reals(i);
   for (real_dependencies::iterator i = reals.begin(), i_end = reals.end(); i != i_end; ++i)
   {
     real_dependency &r = i->second;
     if (r.visited == visit_counter) continue;
-    for (scheme_set::iterator j = r.schemes.begin(), j_end = r.schemes.end(); j != j_end; ++j)
-      if (*j) delete_scheme(*j, NULL);
+    for (proof_scheme const *j: r.schemes) {
+      if (j) delete_scheme(j, NULL);
+    }
     r.schemes.clear();
   }
   // remove reals without any scheme
@@ -422,11 +386,8 @@ preal_vect generate_proof_paths()
   {
     predicated_real real = pending_reals.back();
     pending_reals.pop_back();
-    real_dependency &r = reals[real];
-    for (scheme_set::const_iterator i = r.dependent.begin(),
-         i_end = r.dependent.end(); i != i_end; ++i)
+    for (proof_scheme const *s: reals[real].dependent)
     {
-      proof_scheme const *s = *i;
       if (!s) continue;
       predicated_real u = s->real;
       real_dependency const &t = reals[u];
@@ -441,9 +402,7 @@ preal_vect generate_proof_paths()
   {
     real_dependency &r = i->second;
     r.schemes.erase(NULL);
-    for (scheme_set::iterator j = r.schemes.begin(), j_end = r.schemes.end(); j != j_end; ++j)
-    {
-      proof_scheme const *s = *j;
+    for (proof_scheme const *s: r.schemes) {
       if (s->needed_reals.empty())
         source_schemes.push_back(s);
     }
@@ -459,26 +418,24 @@ preal_vect generate_proof_paths()
       real_dependency &r = i->second;
       std::string var = dump_real(i->first);
       out << "  \"" << var << "\" [label=\"{";
-      for (std::string::const_iterator j = var.begin(), j_end = var.end(); j != j_end; ++j)
+      for (char j: var)
       {
-        switch (*j) {
+        switch (j) {
           case '<': out << "&lt;"; break;
           case '>': out << "&gt;"; break;
           case '|': out << "&#124;"; break;
-          default: out << *j;
+          default: out << j;
         }
       }
       int num_th = 0;
-      for (scheme_set::iterator j = r.schemes.begin(), j_end = r.schemes.end(); j != j_end; ++j)
-        out << "|<" << ++num_th << '>' << typeid(**j).name();
+      for (proof_scheme const *j: r.schemes)
+        out << "|<" << ++num_th << '>' << typeid(*j).name();
       out << "}\"];\n";
       num_th = 0;
-      for (scheme_set::iterator j = r.schemes.begin(), j_end = r.schemes.end(); j != j_end; ++j)
-      {
+      for (proof_scheme const *j: r.schemes) {
         ++num_th;
-        preal_vect v = (*j)->needed_reals;
-        for (preal_vect::const_iterator k = v.begin(), k_end = v.end(); k != k_end; ++k)
-          out << "  \"" << var << "\":" << num_th << " -> \"" << dump_real(*k) << "\";\n";
+        for (predicated_real const &k: j->needed_reals)
+          out << "  \"" << var << "\":" << num_th << " -> \"" << dump_real(k) << "\";\n";
       }
     }
     out << "}\n";
@@ -486,21 +443,18 @@ preal_vect generate_proof_paths()
   }
   // find unreachable outputs
   preal_vect missing_targets;
-  for (preal_set::iterator i = output_reals.begin(),
-       i_end = output_reals.end(); i != i_end; ++i)
-  {
-    real_dependencies::iterator j = reals.find(*i);
+  for (predicated_real const &i: output_reals) {
+    real_dependencies::iterator j = reals.find(i);
     if (j == reals.end())
-      missing_targets.push_back(*i);
+      missing_targets.push_back(i);
   }
   return missing_targets;
 }
 
 RUN_LAST(proof_paths_cleaner) {
   for(real_dependencies::iterator i = reals.begin(), i_end = reals.end(); i != i_end; ++i) {
-    scheme_set &s = i->second.schemes;
-    for(scheme_set::iterator j = s.begin(), j_end = s.end(); j != j_end; ++j)
-      delete *j;
+    for (proof_scheme const *j: i->second.schemes)
+      delete j;
   }
 }
 
@@ -512,8 +466,8 @@ static bool is_usable(node const *n)
 
 static bool fill_hypotheses(property *hyp, preal_vect const &v)
 {
-  for(preal_vect::const_iterator i = v.begin(), end = v.end(); i != end; ++i) {
-    node *n = find_proof(*i);
+  for(predicated_real const &i: v) {
+    node *n = find_proof(i);
     if (!n || !is_usable(n)) return false;
     *(hyp++) = n->get_result();
   }
@@ -681,11 +635,8 @@ void graph_t::populate(property_tree const &targets,
 
   // Look for an expression on which to perform a point-based dichotomy.
   splitting s;
-  for (std::list<logic_node *>::const_iterator i = trees.begin(),
-       i_end = trees.end(); i != i_end; ++i)
-  {
-    (*i)->tree.get_splitting(s);
-  }
+  for (logic_node const *i: trees)
+    i->tree.get_splitting(s);
   unsigned max_pts = 0;
   splitting::value_type const *sv = NULL;
   for (splitting::const_iterator i = s.begin(), i_end = s.end(); i != i_end; ++i)
@@ -697,10 +648,8 @@ void graph_t::populate(property_tree const &targets,
       interval const &b = n->get_result().bnd();
       if (is_singleton(b)) continue;
       nb = 0;
-      for (split_point_mset::const_iterator j = i->second.begin(),
-           j_end = i->second.end(); j != j_end; ++j)
-      {
-        if (lower(b) < j->pt && j->pt < upper(b)) ++nb;
+      for (split_point const &j: i->second) {
+        if (lower(b) < j.pt && j.pt < upper(b)) ++nb;
       }
       if (nb <= max_pts) continue;
     }
@@ -711,12 +660,10 @@ void graph_t::populate(property_tree const &targets,
   {
     uintptr_t ds = 0;
     number prev = number::neg_inf;
-    for (split_point_mset::const_iterator i = sv->second.begin(),
-         i_end = sv->second.end(); i != i_end; ++i)
-    {
-      if (i->pt == prev) continue;
-      ds = fill_splitter(ds, *i);
-      prev = i->pt;
+    for (split_point const &i: sv->second) {
+      if (i.pt == prev) continue;
+      ds = fill_splitter(ds, i);
+      prev = i.pt;
     }
     dichotomy_var dv = { sv->first, ds };
     dichotomy_hint dh = { dvar_vect(1, dv), property_tree(), false };
