@@ -40,7 +40,7 @@ struct proof_scheme
    *       #needed_reals, and @a name should be initialized with #default_name.
    */
   virtual void compute(property const hyps[], property &res, std::string &name) const = 0;
-  virtual ~proof_scheme() {}
+  virtual ~proof_scheme() = default;
   /** Initializes the scheme with the structure of the goal and hypotheses. */
   proof_scheme(predicated_real const &r, preal_vect const &n, char const *d, update u = UPD_SEEK)
     : real(r), needed_reals(n), default_name(d), update_kind(u), visited(0),
@@ -84,7 +84,7 @@ struct scheme_factory
    *         e.g. a more precise one exists.
    */
   virtual proof_scheme *operator()(predicated_real const &r, ast_real_vect const &holes) const = 0;
-  virtual ~scheme_factory() {}
+  virtual ~scheme_factory() = default;
 };
 
 /**
